@@ -75,6 +75,7 @@ static void CairoGD_Raster(unsigned int *raster, int w, int h,
                        double rot, Rboolean interpolate,
                        R_GE_gcontext *gc, NewDevDesc *dd);
 static SEXP CairoGD_Cap(NewDevDesc *dd);
+static int  CairoGD_HoldFlush(NewDevDesc *dd, int level);
 
 /* fake mbcs support for old R versions */
 #if R_GE_version < 4
@@ -107,7 +108,11 @@ Rcairo_font_face Rcairo_fonts[5] = {
 	{ NULL, 0 }
 };
 
+static const cairo_user_data_key_t key;
+
 cairo_font_face_t *Rcairo_set_font_face(int i, const char *file){
+	cairo_font_face_t *c_face;
+	cairo_status_t status;
 	FT_Face face;
 	FT_Error er;
 	FT_CharMap found = 0; 
@@ -143,7 +148,15 @@ cairo_font_face_t *Rcairo_set_font_face(int i, const char *file){
 		er = FT_Set_Charmap( face, found );
 	} 
 
-	return cairo_ft_font_face_create_for_ft_face(face,FT_LOAD_DEFAULT);
+	c_face = cairo_ft_font_face_create_for_ft_face(face,FT_LOAD_DEFAULT);
+	status = cairo_font_face_set_user_data (c_face, &key,
+		face, (cairo_destroy_func_t) FT_Done_Face);
+	if (status) {
+	    cairo_font_face_destroy (c_face);
+	    FT_Done_Face (face);
+	    return NULL;
+	}
+	return c_face;
 }
 
 void Rcairo_set_font(int i, const char *fcname){
@@ -197,6 +210,12 @@ void Rcairo_set_font(int i, const char *fcname){
 }
 #endif
 
+static void set_cf_antialias(cairo_t *cc) {
+	cairo_font_options_t *fo = cairo_font_options_create();
+	cairo_font_options_set_antialias(fo, CAIRO_ANTIALIAS_SUBPIXEL);
+	cairo_set_font_options(cc, fo);
+	cairo_font_options_destroy(fo);
+}
 
 static void Rcairo_setup_font(CairoGDDesc* xd, R_GE_gcontext *gc) {
 	cairo_t *cc = xd->cb->cc;
@@ -208,6 +227,7 @@ static void Rcairo_setup_font(CairoGDDesc* xd, R_GE_gcontext *gc) {
 
 	if (Rcairo_fonts[i].updated || (xd->fontface != gc->fontface)){
 		cairo_set_font_face(cc,Rcairo_fonts[i].face);
+		set_cf_antialias(cc);
 		Rcairo_fonts[i].updated = 0;
 #ifdef JGD_DEBUG
 		  Rprintf("  font face changed to \"%d\" %fpt\n", gc->fontface, gc->cex*gc->ps + 0.5);
@@ -230,7 +250,7 @@ static void Rcairo_setup_font(CairoGDDesc* xd, R_GE_gcontext *gc) {
   if (gc->fontface==2 || gc->fontface==4) wght=CAIRO_FONT_WEIGHT_BOLD;
   
   cairo_select_font_face (cc, Cfontface, slant, wght);
-
+  set_cf_antialias(cc);
 #ifdef JGD_DEBUG
   Rprintf("  font \"%s\" %fpt (slant:%d, weight:%d)\n", Cfontface, gc->cex*gc->ps + 0.5, slant, wght);
 #endif
@@ -402,6 +422,26 @@ static void CairoGD_Deactivate(NewDevDesc *dd)
 	if (xd->cb->activation) xd->cb->activation(xd->cb, 0);
 }
 
+static int  CairoGD_HoldFlush(NewDevDesc *dd, int level)
+{
+	int ol;
+    CairoGDDesc *xd = (CairoGDDesc *) dd->deviceSpecific;
+    if (!xd) return 0;
+	ol = xd->holdlevel;
+    xd->holdlevel += level;
+    if (xd->holdlevel < 0)
+		xd->holdlevel = 0;
+    if (xd->holdlevel == 0) { /* flush */
+		if (xd->cb && xd->cb->sync) /* if a back-end provides sync, we just pass on */
+			xd->cb->sync(xd->cb);
+		else /* otherwise just do cairo-side flush */
+			if (xd->cb && xd->cb->cs) cairo_surface_flush(xd->cb->cs);
+	} else if (ol == 0) { /* first hold */
+        /* could display a wait cursor or something ... */
+    }
+    return xd->holdlevel;
+}
+
 static Rboolean CairoGD_Locator(double *x, double *y, NewDevDesc *dd)
 {
     CairoGDDesc *xd = (CairoGDDesc *) dd->deviceSpecific;
@@ -556,6 +596,9 @@ Rboolean CairoGD_Open(NewDevDesc *dd, CairoGDDesc *xd,  const char *type, int co
 		!strcmp(type,"tif")  || !strcmp(type,"tiff") || !strcmp(type, "raster")) {
 		int alpha_plane = 0;
 		int quality = 0; /* find out if we have quality setting */
+#if R_GE_version >= 9
+		dd->haveLocator = 1; /* no locator on image back-ends */
+#endif
 		if (R_ALPHA(xd->bg) < 255) alpha_plane=1;
 		if (!strcmp(type,"jpeg") || !strcmp(type,"jpg")) {
 			SEXP arg = findArg("quality", aux);
@@ -564,6 +607,10 @@ Rboolean CairoGD_Open(NewDevDesc *dd, CairoGDDesc *xd,  const char *type, int co
 			if (quality<0) quality=0;
 			if (quality>100) quality=100;
 			alpha_plane=0;
+#if R_GE_version >= 9
+			/* JPEG has no tbg */
+			dd->haveTransparentBg = 1;
+#endif
 		}
 		if (!strcmp(type,"tif")  || !strcmp(type,"tiff")) {
 			SEXP arg = findArg("compression", aux);
@@ -588,6 +635,11 @@ Rboolean CairoGD_Open(NewDevDesc *dd, CairoGDDesc *xd,  const char *type, int co
 		xd->cb = Rcairo_new_image_backend(xd->cb, conn, file, type, (int)(w+0.5), (int)(h+0.5), quality, alpha_plane);
 	}
 	else if (!strcmp(type,"pdf") || !strcmp(type,"ps") || !strcmp(type,"postscript") || !strcmp(type,"svg")) {
+#if R_GE_version >= 9
+		/* no locator, no capture */
+		dd->haveLocator = 1;
+		dd->haveCapture = 1;
+#endif
 		/* devices using native units, covert those to points */
 		if (umpl<0) {
 			if (xd->dpix <= 0)
@@ -961,6 +1013,9 @@ void Rcairo_setup_gd_functions(NewDevDesc *dd) {
 	dd->cap = CairoGD_Cap;
 #if R_GE_version >= 8
 	dd->path = CairoGD_Path;
+#if R_GE_version >= 9
+	dd->holdflush = CairoGD_HoldFlush;
+#endif
 #endif
 #endif
 #endif
@@ -1027,6 +1082,7 @@ void Rcairo_backend_init_surface(Rcairo_backend *be) {
 	cairo_select_font_face (cc, "Helvetica",
 			CAIRO_FONT_SLANT_NORMAL,
 			CAIRO_FONT_WEIGHT_NORMAL);
+	set_cf_antialias(cc);
 	cairo_set_font_size (cc, 14);
 #endif
 }
